/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.di;

import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.util.IServiceAccountJwtClient;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubRequestBodyExtractor;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import java.util.Map;

import org.opengroup.osdu.core.common.model.http.AppException;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.context.annotation.RequestScope;

import javax.servlet.http.HttpServletRequest;

@Component
@RequestScope
@ConditionalOnProperty(value = "requestScope.enabled", havingValue = "true", matchIfMissing = true)
@Primary
public class CredentialHeadersProvider implements FactoryBean<DpsHeaders> {

    @Autowired
    private HttpServletRequest httpRequest;

    @Autowired
    private IPubsubRequestBodyExtractor pubsubRequestBodyExtractor;

    @Autowired
    private IServiceAccountJwtClient serviceAccountJwtClient;

    @Override
    public Class<?> getObjectType() {
        return DpsHeaders.class;
    }

    @Override
    public DpsHeaders getObject() throws Exception {

        RequestMethod requestMethod = RequestMethod.valueOf(httpRequest.getMethod());

        switch (requestMethod) {
            case POST:
            case PUT:
            case PATCH:
                return getDpsHeadersForPostPutPatch();

            default: {
                return new DpsHeaders();
            }
        }
    }

    private DpsHeaders getDpsHeadersForPostPutPatch() throws Exception {
        Map<String, String> attributes = this.pubsubRequestBodyExtractor.extractAttributesFromRequestBody();
        try {
            //extract headers from pubsub message
            String dataPartitionId = attributes.get(DpsHeaders.DATA_PARTITION_ID);
            String authToken = this.serviceAccountJwtClient.getIdToken(dataPartitionId);
            attributes.put(DpsHeaders.AUTHORIZATION, authToken);
            return DpsHeaders.createFromMap(attributes);
        } catch (AppException e) {
            throw new Exception("Failed to generate headers for register service.", e);
        }
    }
}